(function($) {

	var kwPopups = [];

	// make sure the popup position is updated on resize
	function handleWindowResize() {
		$.each(kwPopups, function() {
			this.refresh(true);
		});
	}
	$(window).resize(handleWindowResize);

	$.KeywordPopup = function(elm, options) {
		this.$elm = $(elm);
		this.opts = $.extend({}, $.fn.keywordpopup.defaults, options);
      this.$keywordPopup = $(['<div id="ccx-keywordpopup">',
                  '<div class="ccx-keywordpopup-base">',
                  '<div class="ccx-keywordpopup-loading">Loading<br><img src="'+IMWB_CCX_Ajax.plugin_url+'/images/ajax-loader-big.gif"></div>',
                  '</div>',
                  '</div>'].join('')).appendTo(document.body);
			
		this.$inner = this.$keywordPopup.find('div.ccx-keywordpopup-base');
		this.disabled = false;
		this.content = null;
		this.createTime = 0;
		this.init();
	};

	$.KeywordPopup.hideAll = function() {
		$.each(kwPopups, function() {
			this.hide();
		});
	};

	$.KeywordPopup.prototype = {
		init: function() {
			kwPopups.push(this);
			
         // save a reference to the Poshytip object
         this.$elm.data('keywordpopup', this);
		
			// hook element events
			this.$elm.bind({
				'mouseenter.keywordpopup': $.proxy(this.mouseenter, this),
				'touchstart.keywordpopup': $.proxy(this.mouseenter, this),
				'mouseleave.keywordpopup': $.proxy(this.mouseleave, this)
			});
			this.$keywordPopup.hover($.proxy(this.clearTimeouts, this), $.proxy(this.mouseleave, this));
			this.$keywordPopup.bind('touchstart.keywordpopup', function(event){ event.stopPropagation(); });    
			$(document).bind({'touchstart.keywordpopup': $.proxy(this.mouseleave, this)});
		},
		mouseenter: function(e) {
			if (this.disabled)
				return true;

			this.showDelayed();
		},
		mouseleave: function(e) {
			if (this.disabled || this.asyncAnimating && (this.$keywordPopup[0] === e.relatedTarget || jQuery.contains(this.$keywordPopup[0], e.relatedTarget)))
				return true;

			this.hideDelayed();
		},
		show: function() {
			if (this.disabled || this.$keywordPopup.data('active'))
				return;

			this.reset();
			this.update();

			this.display();
		},
		showDelayed: function(timeout) {
			this.clearTimeouts();
			this.showTimeout = setTimeout($.proxy(this.show, this), typeof timeout == 'number' ? timeout : this.opts.showTimeout);
		},
		hide: function() {
			if (this.disabled || !this.$keywordPopup.data('active'))
				return;

			this.display(true);
		},
		hideDelayed: function(timeout) {
			this.clearTimeouts();
			this.hideTimeout = setTimeout($.proxy(this.hide, this), typeof timeout == 'number' ? timeout : this.opts.hideTimeout);
		},
		reset: function() {
			this.$keywordPopup.queue([]).detach().css('visibility', 'hidden').data('active', false);
			this.$inner.find('*').keywordpopup('hide');
			if (this.opts.fade)
				this.$keywordPopup.css('opacity', this.opacity);
			this.asyncAnimating = false;
		},
		update: function() {
			if (this.disabled)
				return;
				
			// update content only if it has been changed since last time
			var self = this;
			
         // If item is more that 30 minutes old then re-fetch it
         if(this.content && ($.now() - this.createTime > 1800000)) {
            this.$inner.empty().append('<div class="ccx-keywordpopup-loading">Loading<br><img src="'+IMWB_CCX_Ajax.plugin_url+'/images/ajax-loader-big.gif"></div>');
            this.content = null;
         }
			
			if(!this.content) {
            var data = { action :'get_contextual_ad', adid: this.$elm.data('ad-num'), keyword: this.$elm.text(), nonce : IMWB_CCX_Ajax.nonce};
            $.ajax({async: true, type: 'POST', url: IMWB_CCX_Ajax.ajaxurl,context: this, data: data,  dataType: 'html'}).done(function(html) {
               this.$inner.empty().append(html);
               this.content = html;
               this.createTime = $.now();
            });
         }
			
         var data1 = { action :'update_keyword_imp', keyword: this.$elm.text(), nonce : IMWB_CCX_Ajax.nonce};
			$.ajax({async: true, type: 'POST',url: IMWB_CCX_Ajax.ajaxurl, data: data1,  dataType: 'html'});
			
			this.refresh(false);
		},
		refresh: function(async) {
			if (this.disabled)
				return;

			if (async) {
				if (!this.$keywordPopup.data('active'))
					return;
				// save current position as we will need to animate
				var currPos = {left: this.$keywordPopup.css('left'), top: this.$keywordPopup.css('top')};
			}

			// reset position to avoid text wrapping, etc.
			this.$keywordPopup.css({left: 0, top: 0}).appendTo(document.body);

			// save default opacity
			if (this.opacity === undefined)
				this.opacity = this.$keywordPopup.css('opacity');
				
         this.kwPopupOuterW = this.$keywordPopup.outerWidth();
         this.kwPopupOuterH = this.$keywordPopup.outerHeight();

			this.calcPos();

			if (async && this.opts.refreshAniDuration) {
				this.asyncAnimating = true;
				var self = this;
				this.$keywordPopup.css(currPos).animate({left: this.pos.l, top: this.pos.t}, this.opts.refreshAniDuration, function() { self.asyncAnimating = false; });
			} else {
				this.$keywordPopup.css({left: this.pos.l, top: this.pos.t});
			}
		},
		display: function(hide) {
			var active = this.$keywordPopup.data('active');
			if (active && !hide || !active && hide)
				return;

			this.$keywordPopup.stop();
			if (this.opts.fade && (hide && this.opts.hideAniDuration || !hide && this.opts.showAniDuration)) {
				var from = {}, to = {};
				if (this.opts.fade) {
					from.opacity = hide ? this.$keywordPopup.css('opacity') : 0;
					to.opacity = hide ? 0 : this.opacity;
				}
				this.$keywordPopup.css(from).animate(to, this.opts[hide ? 'hideAniDuration' : 'showAniDuration']);
			}
			hide ? this.$keywordPopup.queue($.proxy(this.reset, this)) : this.$keywordPopup.css('visibility', 'inherit');
			this.$keywordPopup.data('active', !active);			
		},
		disable: function() {
			this.reset();
			this.disabled = true;
		},
		enable: function() {
			this.disabled = false;
		},
		destroy: function() {
			this.reset();
			this.$keywordPopup.remove();
			delete this.$keywordPopup;
			this.content = null;
			this.$elm.unbind('.keywordpopup').removeData('keywordpopup');
			kwPopups.splice($.inArray(this, kwPopups), 1);
		},
		clearTimeouts: function() {
			if (this.showTimeout) {
				clearTimeout(this.showTimeout);
				this.showTimeout = 0;
			}
			if (this.hideTimeout) {
				clearTimeout(this.hideTimeout);
				this.hideTimeout = 0;
			}
		},
		calcPos: function() {
			var pos = {l: 0, t: 0},
				$win = $(window),
				win = {
					l: $win.scrollLeft(),
					t: $win.scrollTop(),
					w: $win.width(),
					h: $win.height()
				}, xL, xC, xR, yT, yC, yB;

			var elmOffset = this.$elm.offset(),
				elm = {
					l: elmOffset.left,
					t: elmOffset.top,
					w: this.$elm.outerWidth(),
					h: this.$elm.outerHeight()
				};
			xL = elm.l;	// left edge
			xC = xL + Math.floor(elm.w / 2);	// h center
			xR = xL + elm.w;	// right edge
			yT = elm.t;	// top edge
			yC = yT + Math.floor(elm.h / 2); // v center
			yB = yT + elm.h;	// bottom edge

			// keep in viewport - Horizontal
			pos.l = xL;
			if (this.opts.keepInViewport && pos.l < win.l)
				pos.l = win.l+this.opts.viewPortMargin;
         if (this.opts.keepInViewport && pos.l+this.kwPopupOuterW > win.w)
            pos.l = pos.l-((pos.l+this.kwPopupOuterW )-win.w)-this.opts.viewPortMargin;

         // Keep in viewport - Vertical
			pos.t = yT-this.kwPopupOuterH;
			if (this.opts.keepInViewport && pos.t < win.t) 
				pos.t = yB;
				
			this.pos = pos;
		}
	};

	$.fn.keywordpopup = function(options) {
		if (typeof options == 'string') {
			var args = arguments,
				method = options;
			Array.prototype.shift.call(args);
			// unhook live events if 'destroy' is called
			if (method == 'destroy') {
				this.die ?
					this.die('mouseenter.keywordpopup').die('focus.keywordpopup') :
					$(document).undelegate(this.selector, 'mouseenter.keywordpopup').undelegate(this.selector, 'focus.keywordpopup');
			}
			return this.each(function() {
				var keywordpopup = $(this).data('keywordpopup');
				if (keywordpopup && keywordpopup[method])
					keywordpopup[method].apply(keywordpopup, args);
			});
		}

		var opts = $.extend({}, $.fn.keywordpopup.defaults, options);

      // check if we need to hook live events
      if (opts.liveEvents) {
         var handler,
            deadOpts = $.extend({}, opts, { liveEvents: false });

			handler = function() {
				var $this = $(this);
				if (!$this.data('keywordpopup'))
					$this.keywordpopup(deadOpts).keywordpopup('mouseenter');
			};
			// support 1.4.2+ & 1.9+
			this.live ?
				this.live('mouseenter.keywordpopup', handler) :
				$(document).delegate(this.selector, 'mouseenter.keywordpopup', handler);
         return this;
      }
		
		return this.each(function() {
			new $.KeywordPopup(this, opts);
		});
   };

	// default settings
	$.fn.keywordpopup.defaults = {
		showTimeout:		500,		// timeout before showing the tip (in milliseconds 1000 == 1 second)
		hideTimeout:		100,		// timeout before hiding the tip
		liveEvents:		true,		// use live events
		keepInViewport:		true,		// reposition the tooltip if needed to make sure it always appears inside the viewport
		viewPortMargin:   10,      // Margin to use when adjusting popup to fit in viewport, so it doesnt butt up against the browser edge
      fade:            true,		// use fade animation
      slide:            true,		// use slide animation
		slideOffset:      8,		// slide animation offset
		showAniDuration:   200,		// show animation duration - set to 0 if you don't want show animation
		hideAniDuration:   200,		// hide animation duration - set to 0 if you don't want hide animation
		refreshAniDuration:   200		// refresh animation duration - set to 0 if you don't want animation when updating the tooltip asynchronously
	};

})(jQuery);